==> snowflake_data_validation-0.0.1.tar.gz: snowflake_data_validation-0.0.1/src/snowflake/snowflake_data_validation/utils/arguments_manager_base.py <==
# Copyright 2025 Snowflake Inc.
# SPDX-License-Identifier: Apache-2.0
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import uuid

from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

from snowflake.snowflake_data_validation.configuration.configuration_loader import (
    ConfigurationLoader,
)
from snowflake.snowflake_data_validation.configuration.model.configuration_model import (
    ConfigurationModel,
)
from snowflake.snowflake_data_validation.connector.connector_base import (
    ConnectorBase,
)
from snowflake.snowflake_data_validation.utils.base_output_handler import (
    OutputHandlerBase,
)
from snowflake.snowflake_data_validation.utils.console_output_handler import (
    ConsoleOutputHandler,
)
from snowflake.snowflake_data_validation.utils.constants import (
    COLUMN_DATATYPES_MAPPING_NAME_FORMAT,
    COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT,
    COLUMN_METRICS_TEMPLATE_NAME_FORMAT,
    Platform,
)
from snowflake.snowflake_data_validation.utils.context import Context
from snowflake.snowflake_data_validation.utils.helpers import (
    _dump_and_write_yaml_template,
    copy_templates_to_temp_dir,
    create_temp_dir,
)
from snowflake.snowflake_data_validation.utils.logging_config import (
    relocate_log_file,
)
from snowflake.snowflake_data_validation.utils.model.templates_loader_manager import (
    TemplatesLoaderManager,
)


@dataclass
class ValidationEnvironmentObject:

    """Container for all components needed by the orchestrator."""

    source_connector: ConnectorBase
    target_connector: ConnectorBase
    context: Context

    def cleanup(self):
        """Clean up connections and resources."""
        if hasattr(self.source_connector, "close"):
            self.source_connector.close()
        if hasattr(self.target_connector, "close"):
            self.target_connector.close()


class ArgumentsManagerBase(ABC):

    """Abstract base class for dialect-specific argument managers."""

    def __init__(self, source_platform: Platform, target_platform: Platform):
        """Initialize the arguments manager with source and target platform identifiers.

        Args:
            source_platform (Platform): The source platform enum (e.g., Platform.SNOWFLAKE, Platform.SQLSERVER).
            target_platform (Platform): The target platform enum (e.g., Platform.SNOWFLAKE, Platform.SQLSERVER).

        """
        self.source_platform = source_platform
        self.target_platform = target_platform

    @abstractmethod
    def create_validation_environment_from_config(
        self,
        config_model: ConfigurationModel,
        data_validation_config_file: str,
        output_handler: Optional[ConsoleOutputHandler] = None,
    ) -> ValidationEnvironmentObject:
        """Create a complete validation environment from configuration model.

        Args:
            config_model: The loaded configuration model
            data_validation_config_file: Path to the config file
            output_handler: Optional output handler

        Returns:
            ValidationEnvironmentObject: Validation environment ready to run

        Raises:
            typer.BadParameter: If configuration is invalid
            ValueError: If required connections are missing

        """
        pass

    @abstractmethod
    def setup_source_connection(self, **kwargs) -> ConnectorBase:
        """Set up source database connection and return connector.

        Args:
            **kwargs: Dialect-specific connection parameters

        Returns:
            ConnectorBase: Configured source connector

        """
        pass

    @abstractmethod
    def setup_target_connection(self, **kwargs) -> ConnectorBase:
        """Set up target database connection and return connector.

        Args:
            **kwargs: Dialect-specific connection parameters

        Returns:
            ConnectorBase: Configured target connector

        """
        pass

    @abstractmethod
    def get_source_templates_path(self) -> str:
        """Get the path to dialect-specific templates.

        Returns:
            str: Path to source templates directory

        """
        pass

    @abstractmethod
    def get_target_templates_path(self) -> str:
        """Get the path to target dialect-specific templates.

        Returns:
            str: Path to target templates directory

        """
        pass

    @abstractmethod
    def get_default_datatypes_mapping(self) -> dict[str, str]:
        """Get default datatype mappings for this dialect.

        Returns:
            dict[str, str]: Default datatype mappings

        """
        pass

    @property
    @abstractmethod
    def is_snowflake_to_snowflake(self) -> bool:
        """Check if this is a Snowflake-to-Snowflake validation scenario.

        Returns:
            bool: True if both source and target are Snowflake

        """
        pass

    def setup_validation_environment(
        self,
        source_connector: ConnectorBase,
        target_connector: ConnectorBase,
        data_validation_config_file: str,
        output_directory_path: Optional[str] = None,
        output_handler: Optional[OutputHandlerBase] = None,
        datatypes_mappings: Optional[dict[str, str]] = None,
    ) -> ValidationEnvironmentObject:
        """Set up the validation environment with context and all necessary components.

        Args:
            source_connector: Configured source connector
            target_connector: Configured target connector
            data_validation_config_file: Path to validation configuration
            output_directory_path: Optional path for output directory (reports, logs, etc.)
            output_handler: Optional custom output handler
            datatypes_mappings: Optional datatype mappings

        Returns:
            ValidationEnvironmentObject: Complete environment ready for orchestrator

        """
        # Set up temporary directories for execution
        execution_temp_dir = create_temp_dir()
        output_directory_path_ = (
            output_directory_path
            if output_directory_path
            else os.path.join(execution_temp_dir, "reports")
        )
        templates_temp_dir_path = os.path.join(execution_temp_dir, "templates")

        for directory in [output_directory_path_, templates_temp_dir_path]:
            os.makedirs(directory, exist_ok=True)

        # Relocate log file to the configured output directory
        relocate_log_file(output_directory_path_)

        # Get templates paths
        source_templates_path = self.get_source_templates_path()
        target_templates_path = self.get_target_templates_path()

        copy_templates_to_temp_dir(
            source_templates_path, target_templates_path, templates_temp_dir_path
        )

        # Set up output handler
        if not output_handler:
            output_handler = ConsoleOutputHandler()

        # Generate run metadata
        run_id = str(uuid.uuid4())
        run_start_time = datetime.now(timezone.utc).strftime("%Y%m%dT%H%M%S")
        # Load configuration model
        configuration = self.load_configuration(data_validation_config_file)

        source_template_loader_manager = TemplatesLoaderManager(
            templates_directory_path=Path(self.get_source_templates_path()),
            platform=self.source_platform,
        )

        target_template_loader_manager = TemplatesLoaderManager(
            templates_directory_path=Path(self.get_target_templates_path()),
            platform=self.target_platform,
        )

        context = Context(
            configuration=configuration,
            report_path=output_directory_path_,
            templates_dir_path=templates_temp_dir_path,
            source_platform=self.source_platform,
            target_platform=self.target_platform,
            custom_output_handler=output_handler,
            run_id=run_id,
            run_start_time=run_start_time,
            source_templates=source_template_loader_manager,
            target_templates=target_template_loader_manager,
            datatypes_mappings=datatypes_mappings,
        )

        return ValidationEnvironmentObject(
            source_connector=source_connector,
            target_connector=target_connector,
            context=context,
        )

    def load_configuration(
        self, data_validation_config_file: str
    ) -> ConfigurationModel:
        """Load the configuration model from the specified YAML file.

        Args:
            data_validation_config_file (str): Path to the configuration YAML file.

        Returns:
            ConfigurationModel: Loaded configuration model.

        """
        config_loader = ConfigurationLoader(Path(data_validation_config_file))
        return config_loader.get_configuration_model()

    def dump_and_write_yaml_templates(
        self, source: str, templates_directory: str
    ) -> None:
        """Dump and write YAML templates to specified output files.

        This function iterates over the list of template paths,
        dumps their content, and writes it to the specified output files.

        Args:
            source (str): The source platform identifier (e.g., 'snowflake', 'sqlserver').
            templates_directory (str): The directory where the templates will be written.

        """
        templates_path = self.get_source_templates_path()
        target_templates_path = self.get_target_templates_path()
        templates = []
        # dump from source
        # Collect templates from source if not Snowflake
        if source != Platform.SNOWFLAKE.value:
            templates += [
                os.path.join(
                    templates_path,
                    COLUMN_DATATYPES_MAPPING_NAME_FORMAT.format(platform=source),
                ),
                os.path.join(
                    templates_path,
                    COLUMN_METRICS_TEMPLATE_NAME_FORMAT.format(platform=source),
                ),
                os.path.join(
                    templates_path,
                    COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT.format(
                        platform=source
                    ),
                ),
            ]
        # Always collect Snowflake templates from target
        templates += [
            os.path.join(
                target_templates_path,
                COLUMN_METRICS_TEMPLATE_NAME_FORMAT.format(
                    platform=Platform.SNOWFLAKE.value
                ),
            ),
            os.path.join(
                target_templates_path,
                COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT.format(
                    platform=Platform.SNOWFLAKE.value
                ),
            ),
        ]

        for template in templates:
            _dump_and_write_yaml_template(
                template, output_directory=templates_directory
            )

==> snowflake_data_validation-0.0.5.tar.gz: snowflake_data_validation-0.0.5/src/snowflake/snowflake_data_validation/utils/arguments_manager_base.py <==
# Copyright 2025 Snowflake Inc.
# SPDX-License-Identifier: Apache-2.0
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import uuid

from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

import yaml

from snowflake.snowflake_data_validation.configuration.configuration_loader import (
    ConfigurationLoader,
)
from snowflake.snowflake_data_validation.configuration.model.configuration_model import (
    ConfigurationModel,
)
from snowflake.snowflake_data_validation.connector.connector_base import (
    ConnectorBase,
)
from snowflake.snowflake_data_validation.utils.base_output_handler import (
    OutputHandlerBase,
)
from snowflake.snowflake_data_validation.utils.console_output_handler import (
    ConsoleOutputHandler,
)
from snowflake.snowflake_data_validation.utils.constants import (
    COLUMN_DATATYPES_MAPPING_NAME_FORMAT,
    COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT,
    COLUMN_METRICS_TEMPLATE_NAME_FORMAT,
    Platform,
)
from snowflake.snowflake_data_validation.utils.context import Context
from snowflake.snowflake_data_validation.utils.helper import Helper
from snowflake.snowflake_data_validation.utils.logging_config import (
    relocate_log_file,
)
from snowflake.snowflake_data_validation.utils.model.templates_loader_manager import (
    TemplatesLoaderManager,
)


@dataclass
class ValidationEnvironmentObject:

    """Container for all components needed by the orchestrator."""

    source_connector: ConnectorBase
    target_connector: ConnectorBase
    context: Context

    def cleanup(self):
        """Clean up connections and resources."""
        if hasattr(self.source_connector, "close"):
            self.source_connector.close()
        if hasattr(self.target_connector, "close"):
            self.target_connector.close()


class ArgumentsManagerBase(ABC):

    """Abstract base class for dialect-specific argument managers."""

    def __init__(self, source_platform: Platform, target_platform: Platform):
        """Initialize the arguments manager with source and target platform identifiers.

        Args:
            source_platform (Platform): The source platform enum (e.g., Platform.SNOWFLAKE, Platform.SQLSERVER).
            target_platform (Platform): The target platform enum (e.g., Platform.SNOWFLAKE, Platform.SQLSERVER).

        """
        self.source_platform = source_platform
        self.target_platform = target_platform

    @abstractmethod
    def create_validation_environment_from_config(
        self,
        config_model: ConfigurationModel,
        data_validation_config_file: str,
        output_handler: Optional[ConsoleOutputHandler] = None,
    ) -> ValidationEnvironmentObject:
        """Create a complete validation environment from configuration model.

        Args:
            config_model: The loaded configuration model
            data_validation_config_file: Path to the config file
            output_handler: Optional output handler

        Returns:
            ValidationEnvironmentObject: Validation environment ready to run

        Raises:
            typer.BadParameter: If configuration is invalid
            ValueError: If required connections are missing

        """
        pass

    @abstractmethod
    def setup_source_connection(self, **kwargs) -> ConnectorBase:
        """Set up source database connection and return connector.

        Args:
            **kwargs: Dialect-specific connection parameters

        Returns:
            ConnectorBase: Configured source connector

        """
        pass

    @abstractmethod
    def setup_target_connection(self, **kwargs) -> ConnectorBase:
        """Set up target database connection and return connector.

        Args:
            **kwargs: Dialect-specific connection parameters

        Returns:
            ConnectorBase: Configured target connector

        """
        pass

    @abstractmethod
    def get_source_templates_path(self) -> str:
        """Get the path to dialect-specific templates.

        Returns:
            str: Path to source templates directory

        """
        pass

    @abstractmethod
    def get_target_templates_path(self) -> str:
        """Get the path to target dialect-specific templates.

        Returns:
            str: Path to target templates directory

        """
        pass

    @property
    @abstractmethod
    def is_snowflake_to_snowflake(self) -> bool:
        """Check if this is a Snowflake-to-Snowflake validation scenario.

        Returns:
            bool: True if both source and target are Snowflake

        """
        pass

    def setup_validation_environment(
        self,
        source_connector: ConnectorBase,
        target_connector: ConnectorBase,
        data_validation_config_file: str,
        output_directory_path: Optional[str] = None,
        output_handler: Optional[OutputHandlerBase] = None,
    ) -> ValidationEnvironmentObject:
        """Set up the validation environment with context and all necessary components.

        Args:
            source_connector: Configured source connector
            target_connector: Configured target connector
            data_validation_config_file: Path to validation configuration
            output_directory_path: Optional path for output directory (reports, logs, etc.)
            output_handler: Optional custom output handler

        Returns:
            ValidationEnvironmentObject: Complete environment ready for orchestrator

        """
        # Set up temporary directories for execution
        execution_temp_dir = Helper.create_temp_dir()
        output_directory_path_ = (
            output_directory_path
            if output_directory_path
            else os.path.join(execution_temp_dir, "reports")
        )
        templates_temp_dir_path = os.path.join(execution_temp_dir, "templates")

        for directory in [output_directory_path_, templates_temp_dir_path]:
            os.makedirs(directory, exist_ok=True)

        # Relocate log file to the configured output directory
        relocate_log_file(output_directory_path_)

        # Get templates paths
        source_templates_path = self.get_source_templates_path()
        target_templates_path = self.get_target_templates_path()

        Helper.copy_templates_to_temp_dir(
            source_templates_path, target_templates_path, templates_temp_dir_path
        )

        # Set up output handler
        if not output_handler:
            output_handler = ConsoleOutputHandler()

        # Generate run metadata
        run_id = str(uuid.uuid4())
        run_start_time = datetime.now(timezone.utc).strftime("%Y%m%dT%H%M%S")
        # Load configuration model
        configuration = self.load_configuration(data_validation_config_file)

        source_templates_dir_path = Path(self.get_source_templates_path())

        source_template_loader_manager = TemplatesLoaderManager(
            templates_directory_path=source_templates_dir_path,
            platform=self.source_platform,
        )

        target_template_loader_manager = TemplatesLoaderManager(
            templates_directory_path=Path(self.get_target_templates_path()),
            platform=self.target_platform,
        )

        datatypes_file_path = source_templates_dir_path.joinpath(
            COLUMN_DATATYPES_MAPPING_NAME_FORMAT.format(
                source_platform=self.source_platform.value,
                target_platform=self.target_platform.value,
            )
        )

        datatypes_mappings = Helper.load_datatypes_mapping_templates_from_yaml(
            yaml_path=datatypes_file_path
        )

        context = Context(
            configuration=configuration,
            report_path=output_directory_path_,
            templates_dir_path=templates_temp_dir_path,
            source_platform=self.source_platform,
            target_platform=self.target_platform,
            custom_output_handler=output_handler,
            run_id=run_id,
            run_start_time=run_start_time,
            source_templates=source_template_loader_manager,
            target_templates=target_template_loader_manager,
            datatypes_mappings=datatypes_mappings,
        )

        return ValidationEnvironmentObject(
            source_connector=source_connector,
            target_connector=target_connector,
            context=context,
        )

    def load_configuration(
        self, data_validation_config_file: str
    ) -> ConfigurationModel:
        """Load the configuration model from the specified YAML file.

        Args:
            data_validation_config_file (str): Path to the configuration YAML file.

        Returns:
            ConfigurationModel: Loaded configuration model.

        """
        config_loader = ConfigurationLoader(Path(data_validation_config_file))
        return config_loader.get_configuration_model()

    def dump_and_write_yaml_templates(
        self, source: str, templates_directory: str
    ) -> None:
        """Dump and write YAML templates to specified output files.

        This function iterates over the list of template paths,
        dumps their content, and writes it to the specified output files.

        Args:
            source (str): The source platform identifier (e.g., 'snowflake', 'sqlserver').
            templates_directory (str): The directory where the templates will be written.

        """
        templates_path = self.get_source_templates_path()
        target_templates_path = self.get_target_templates_path()
        templates = []
        # dump from source
        # Collect templates from source if not Snowflake
        if source != Platform.SNOWFLAKE.value:
            templates += [
                os.path.join(
                    templates_path,
                    COLUMN_DATATYPES_MAPPING_NAME_FORMAT.format(
                        source_platform=source, target_platform=Platform.SNOWFLAKE.value
                    ),
                ),
                os.path.join(
                    templates_path,
                    COLUMN_METRICS_TEMPLATE_NAME_FORMAT.format(platform=source),
                ),
                os.path.join(
                    templates_path,
                    COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT.format(
                        platform=source
                    ),
                ),
            ]
        # Always collect Snowflake templates from target
        templates += [
            os.path.join(
                target_templates_path,
                COLUMN_METRICS_TEMPLATE_NAME_FORMAT.format(
                    platform=Platform.SNOWFLAKE.value
                ),
            ),
            os.path.join(
                target_templates_path,
                COLUMN_DATATYPES_NORMALIZATION_TEMPLATES_NAME_FORMAT.format(
                    platform=Platform.SNOWFLAKE.value
                ),
            ),
        ]

        for template in templates:
            self._dump_and_write_yaml_template(
                template, output_directory=templates_directory
            )

    @staticmethod
    def _dump_and_write_yaml_template(template_path: str, output_directory: str) -> str:
        """Dump the content of a YAML template file to a new file.

        Args:
            template_path (str): The path to the YAML template file.
            output_directory (str): The directory where the output file will be written.

        Returns:
            str: The path to the output file where the content was written.

        """
        with open(template_path, encoding="utf-8") as f:
            content = yaml.safe_load(f)
        output_file = os.path.join(output_directory, os.path.basename(template_path))
        with open(output_file, "w", encoding="utf-8") as out_f:
            yaml.dump(content, out_f, default_flow_style=False)
        return str(output_file)
